@@ -3,25 +3,32 @@
 # ============================================================
 
 # Pearson correlation matrix between columns of X, Y
-# http://stackoverflow.com/questions/5888287/running-cor-or-any-variant-over-a-sparse-matrix-in-r
+# https://stackoverflow.com/questions/5888287/running-cor-or-any-variant-over-a-sparse-matrix-in-r
 #
-# covmat uses E[(X-muX)'(Y-muY)] = E[X'Y] - muX'muY
-# with sample correction n/(n-1) this leads to cov = ( X'Y - n*muX'muY ) / (n-1)
+# covmat uses:
+# E[(X-muX)'(Y-muY)] = E[X'Y] - muX'muY
 #
-# the sd in the case Y!=NULL uses E[X-mu]^2 = E[X^2]-mu^2
-# with sample correction n/(n-1) this leads to sd^2 = ( X^2 - n*mu^2 ) / (n-1)
+# with sample correction:
+# n/(n-1) this leads to cov = ( X'Y - n*muX'muY ) / (n-1)
 #
-# Note that results larger than 1e4 x 1e4 will become very slow, because the resulting matrix is not sparse anymore. 
+# the sd in the case Y!=NULL uses:
+# E[X-mu]^2 = E[X^2]-mu^2
+#
+# with sample correction:
+# n/(n-1) this leads to sd^2 = ( X^2 - n*mu^2 ) / (n-1)
+#
+# Note that results larger than 1e4 x 1e4 will become very slow 
+# because the resulting matrix is not sparse anymore. 
 
 corSparse <- function(X, Y = NULL, cov = FALSE) {
 
-	X <- as(X,"dgCMatrix")
+	X <- as(X,"dMatrix")
 	n <- nrow(X)
 	muX <- colMeans(X)
 	
 	if (!is.null(Y)) {
 		stopifnot( nrow(X) == nrow(Y) )
-		Y <- as(Y,"dgCMatrix")
+		Y <- as(Y,"dMatrix")
 		muY <- colMeans(Y)
 		covmat <- ( as.matrix(crossprod(X,Y)) - n*tcrossprod(muX,muY) ) / (n-1)
 		sdvecX <- sqrt( (colSums(X^2) - n*muX^2) / (n-1) )
@@ -63,17 +70,21 @@ corSparse <- function(X, Y = NULL, cov = FALSE) {
 	norm2 <- function(x,s) { drop(crossprod(x^2,s)) ^ 0.5 }
 	# Alternatively take 1-norm
 	norm1 <- function(x,s) { drop(crossprod(abs(x),s)) }
+	# Alternatively, the norm for a normalized Laplacian
+	normL <- function(x,s) { abs(drop(crossprod(x,s))) ^ 0.5 } 
 
 cosSparse <- function(X, Y = NULL, norm = norm2 , weight = NULL) {
 
-	X <- as(X,"dgCMatrix")
+	X <- as(X,"dMatrix")
+	nX <- X@Dimnames
 	if (!is.null(Y)) {
 		stopifnot( nrow(X) == nrow(Y) )
-		Y <- as(Y,"dgCMatrix")
+		Y <- as(Y,"dMatrix")
+		nY <- Y@Dimnames
 	}
 
 	if (!is.null(weight)) {	
-		Nx <- ncol(X)
+	  Nx <- ncol(X)
 		Sx <- rowSums(abs(X))
 		Wx <- Diagonal( x = match.fun(weight)(Sx,Nx) )
 		X <- Wx %*% X
@@ -88,9 +99,11 @@ cosSparse <- function(X, Y = NULL, norm = norm2 , weight = NULL) {
 	S <- rep(1,nrow(X))			
 	N <- Diagonal( x = match.fun(norm)(X,S)^-1 )
 	X <- X %*% N
+	X@Dimnames <- nX
 	if (!is.null(Y)) {
 		N <- Diagonal( x = match.fun(norm)(Y,S)^-1 )
 		Y <- Y %*% N
+		Y@Dimnames <- nY
 		return(crossprod(X,Y))	
 	} else {
 		return(crossprod(X))
@@ -100,10 +113,10 @@ cosSparse <- function(X, Y = NULL, norm = norm2 , weight = NULL) {
 
 cosMissing <- function(X, availX, Y = NULL, availY = NULL, norm = norm2 , weight = NULL) {
 
-	X <- as(X,"dgCMatrix")
+	X <- as(X,"dMatrix")
 	if (!is.null(Y)) {
 		stopifnot( nrow(X) == nrow(Y) )
-		Y <- as(Y,"dgCMatrix")
+		Y <- as(Y,"dMatrix")
 	}
 
 	if (!is.null(weight)) {	
@@ -139,13 +152,13 @@ cosRow <- function(X, rowGroup, Y = NULL, norm = norm2 , weight = NULL) {
 	if (is.vector(rowGroup)) { 
 		rowGroup <- t(ttMatrix(rowGroup)$M*1) 
 	} else {
-		rowGroup <- as(rowGroup,"dgCMatrix")
+		rowGroup <- as(rowGroup,"dMatrix")
 	}
 
-	X <- as(X,"dgCMatrix")
+	X <- as(X,"dMatrix")
 	if (!is.null(Y)) {
 		stopifnot( nrow(X) == nrow(Y) )
-		Y <- as(Y,"dgCMatrix")
+		Y <- as(Y,"dMatrix")
 	}
 
 	if (!is.null(weight)) {	
@@ -181,11 +194,11 @@ cosRow <- function(X, rowGroup, Y = NULL, norm = norm2 , weight = NULL) {
 
 cosCol <- function(X, colGroupX, Y = NULL, colGroupY = NULL, norm = norm2 ) {
 	
-	X <- as(X,"dgCMatrix")
+	X <- as(X,"dMatrix")
 	if (is.vector(colGroupX)) {
 		colGroupX <- ttMatrix(colGroupX)$M * 1 
 	} else {
-		colGroupX <- as(colGroupX,"dgCMatrix")
+		colGroupX <- as(colGroupX,"dMatrix")
 	}
 
 	S <- rep(1,nrow(X))
@@ -196,11 +209,11 @@ cosCol <- function(X, colGroupX, Y = NULL, colGroupY = NULL, norm = norm2 ) {
 	if (!is.null(Y)) {
 		stopifnot( nrow(X) == nrow(Y) )
 		stopifnot(!is.null(colGroupY))
-		Y <- as(Y,"dgCMatrix")
+		Y <- as(Y,"dMatrix")
 		if (is.vector(colGroupY)) { 
 			colGroupY <- ttMatrix(colGroupY)$M * 1 
 		} else {
-			colGroupX <- as(colGroupX,"dgCMatrix")
+			colGroupX <- as(colGroupX,"dMatrix")
 		}	
 
 		Freq <- tcrossprod(Y,colGroupY)
@@ -226,11 +239,13 @@ cosCol <- function(X, colGroupX, Y = NULL, colGroupY = NULL, norm = norm2 ) {
 	# good old pearson residuals
 	res <- function(o,e) { (o-e) / sqrt(e) }
 
-# WATCH OUT! it might not always be the right decision to ignore the cases in which O=zero!!! e.g. residuals should not be zero then (but negative!!!).
+# WATCH OUT! it might not always be the right decision 
+# to ignore the cases in which observed is zero!!! 
+# e.g. residuals should not be zero then (but negative!!!).
 
 assocSparse <- function(X, Y = NULL, method = res, N = nrow(X), sparse = TRUE) {
 
-	X <- as(X,"ngCMatrix")*1
+	X <- as(X,"nMatrix")*1
 	Fx <- colSums(X)
 
 	# observed coocurrences "O"
@@ -238,12 +253,15 @@ assocSparse <- function(X, Y = NULL, method = res, N = nrow(X), sparse = TRUE) {
 		O <- crossprod(X)
 		} else {
 		stopifnot( nrow(X) == nrow(Y) )
-		Y <- as(Y,"ngCMatrix")*1
+		Y <- as(Y,"nMatrix")*1
 		Fy <- colSums(Y)
 		O <- crossprod(X,Y)
 	}
 
-	# The trick to keep things sparse here is to not compute anything when O is zero. In most practical situations this seems to be fine, but note that in cases in which the expectation is high, though observed is zero this might lead to large discrepancies.
+	# The trick to keep things sparse here is to not compute anything when observed is zero.
+	# In most practical situations this seems to be fine, 
+	# but note that in cases in which the expectation is high, though observed is zero
+	# this might lead to large discrepancies.
 	if (sparse) {	
 		R <- as(O,"nMatrix")/N
 		Fx <- Diagonal( x = Fx )
@@ -256,7 +274,9 @@ assocSparse <- function(X, Y = NULL, method = res, N = nrow(X), sparse = TRUE) {
 		}	
 		R@x <- match.fun(method)(O@x,E@x)
 
-	# this is the easy non-sparse method. Note that the result will be non-sparse, so this is not feasable for very large datasets
+	# this is the easy non-sparse method. 
+	# Note that the result will be non-sparse, 
+	# so this is not feasible for very large data sets
 	} else {
 		if (is.null(Y)) {
 			E <- tcrossprod(Fx)/N
@@ -270,18 +290,24 @@ assocSparse <- function(X, Y = NULL, method = res, N = nrow(X), sparse = TRUE) {
 
 # still TO DO: assoc.missing
 
-# The following is a version of assoc for cases in which the columns form groups. Typically found in case of a large collection of nominal variables (e.g. WALS) with an index matrix. If you want to establish the association between the variables, you'd need this version to get the expectation right.
+# The following is a version of assoc for cases in which the 
+# columns form groups. Typically found in case of a large collection of 
+# nominal variables (e.g. WALS) with an index matrix. If you want to 
+# establish the association between the variables, you'd need this version 
+# to get the expectation right.
 
-# colGroupX is either a sparse Matrix with the same number of columns as X, rows represent grouping. Or a grouping vector of length ncol(X), specifying group indices.
+# colGroupX is either a sparse Matrix with the same number of columns as X,
+# rows represent grouping. Or a grouping vector of length ncol(X), 
+# specifying group indices.
 
 assocCol <- function(X, colGroupX, Y = NULL, colGroupY = NULL, method = res, sparse = TRUE) {
 
-	X <- as(X,"dgCMatrix")
+	X <- as(X,"dMatrix")
 	
 	if (is.vector(colGroupX)) { 
 		colGroupX <- ttMatrix(colGroupX)$M*1 
 	} else {
-		colGroupX <- as(colGroupX,"dgCMatrix")	
+		colGroupX <- as(colGroupX,"dMatrix")	
 	}	
 	Gx <- crossprod(colGroupX)
 	
@@ -298,12 +324,12 @@ assocCol <- function(X, colGroupX, Y = NULL, colGroupY = NULL, method = res, spa
 		stopifnot( nrow(X) == nrow(Y) )
 		stopifnot(!is.null(colGroupY))
 
-		Y <- as(Y,"dgCMatrix")
+		Y <- as(Y,"dMatrix")
 		
 		if (is.vector(colGroupY)) { 
 			colGroupY <- ttMatrix(colGroupY)$M*1
 		} else {
-			colGroupY <- as(colGroupY,"dgCMatrix")
+			colGroupY <- as(colGroupY,"dMatrix")
 		}		
 		Gy <- crossprod(colGroupY)
 
@@ -313,10 +339,14 @@ assocCol <- function(X, colGroupX, Y = NULL, colGroupY = NULL, method = res, spa
 		O <- crossprod(X, Y)
 	}
 
-	# in a sparse way: tricky to do, but possible. Ignore items where Observed is zero. Not that this is an approximation that might lead to somewhat unexpected results in specific situations.
+	# in a sparse way: tricky to do, but possible. 
+	# Ignore items where Observed is zero. 
+	# Note that this is an approximation that might lead to somewhat unexpected 
+	# results in specific situations.
 	if (sparse) {	
 
-		# Number of cases per block. To keep this sparse needs some tricks (unfold-refold)	
+		# Number of cases per block.
+	  # To keep this sparse needs some tricks (unfold-refold)	
 		if (is.null(Y)) {			
 			N <- unfoldBlockMatrix(O, colGroupX)	
 		} else {			
@@ -337,7 +367,9 @@ assocCol <- function(X, colGroupX, Y = NULL, colGroupY = NULL, method = res, spa
 		result <- O
 		result@x <- match.fun(method)(O@x,E@x)	
 
-	# in a non-sparse way: this is much more elegant! But the result is not sparse, so this is not feasable for very large datasets.
+	# in a non-sparse way: this is much more elegant!
+	# But the result is not sparse, so this is not feasible
+	# for very large data sets.
 	} else {
 		if (is.null(Y)) {
 			E <- crossprod(X %*% Gx %*% Fx)
@@ -349,18 +381,21 @@ assocCol <- function(X, colGroupX, Y = NULL, colGroupY = NULL, method = res, spa
 	return(result)
 }
 
-# same as above here, but for groups of rows. E.g. with WALS when looking at the similarity between languages, given the expectation of groups (features) as 1/nr.of.values 
+# same as above here, but for groups of rows. 
+# E.g. with WALS when looking at the similarity between languages, 
+# given the expectation of groups (features) as 1/nr.of.values 
 
-# rowGroup is a sparse matrix with same number of rows as X, or a vector of length nrow(X) specifying the grouping of the rows
+# rowGroup is a sparse matrix with same number of rows as X, 
+# or a vector of length nrow(X) specifying the grouping of the rows
 
 assocRow <- function(X, rowGroup, Y = NULL, method = res) {
 
-	X <- as(X,"dgCMatrix")
+	X <- as(X,"dMatrix")
 	
 	if (is.vector(rowGroup)) {
 		rowGroup <- t(ttMatrix(rowGroup)$M*1)
 	} else {
-		rowGroup <- as(rowGroup,"dgCMatrix")
+		rowGroup <- as(rowGroup,"dMatrix")
 	}
 	W <- Diagonal( x = rowSums(tcrossprod(rowGroup))^-0.5 )
 
@@ -369,7 +404,7 @@ assocRow <- function(X, rowGroup, Y = NULL, method = res) {
 		E <- crossprod(t(rowGroup) %*% W %*% X)
 	} else {
 		stopifnot( nrow(X) == nrow(Y) )
-		Y <- as(Y,"dgCMatrix")
+		Y <- as(Y,"dMatrix")
 		O <- crossprod(X,Y)
 		E <- crossprod(t(rowGroup) %*% W %*% X, t(rowGroup) %*% W %*% Y)
 	}
