@@ -44,13 +44,13 @@ splitTable <- function(	data,
   
 	OV <- tt[[1]]$M
 	for (i in tt[-1]) {
-		OV <- rBind(OV,i$M)
+		OV <- rbind(OV,i$M)
 	}
 	OV <- t(OV)
 	
 	# the following approach is slow... strange...
 	# http://www.r-bloggers.com/the-rbinding-race-for-vs-do-call-vs-rbind-fill/
-	# OV <- t(do.call(rBind,sapply(tt,function(x){x[[1]]})))
+	# OV <- t(do.call(rbind,sapply(tt,function(x){x[[1]]})))
 
   cls <- ncol(data)
   
@@ -58,7 +58,8 @@ splitTable <- function(	data,
 	nrValues <- sapply(tt,function(x){length(x$rownames)})
 	AV <- ttMatrix(rep.int(1:cls,nrValues))$M
 	
-	# we need some variable names to make value names, but entity names might be NULL
+	# we need some variable names to make value names, 
+	# but entity names might be NULL
 
 	if (is.null(attributes)) {
 		attributes <- paste("X", 1:cls, sep="")
@@ -80,88 +81,84 @@ splitTable <- function(	data,
 					))
 }
 
-# make unigram and bigram matrices from a vector of strings
+# make ngram matrices from a vector of strings
 
 splitStrings <- function(	strings,
 							sep = "",
-							bigrams = TRUE,
+							n = 2,
 							boundary = TRUE,
-							bigram.binder = "",
-							gap.symbol = "\u00B7",
+							ngram.binder = "",
 							left.boundary = "#",
 							right.boundary = "#",
 							simplify = FALSE
 							) {
-	if (bigrams) {
-		gap.length <- 1
-		originals <- strings
-		if (boundary) {
-			strings <- paste(left.boundary, strings, right.boundary, sep = sep)
-		} 
-	} else {
-		gap.length <- 0
+
+  # add boundary symbols
+	originals <- strings
+	if (n > 1 && boundary) {
+		strings <- paste(left.boundary, strings, right.boundary, sep = sep)
 	}
 
 	# SW: Segments x Strings ("Words")
-	tmp <- pwMatrix(strings, sep=sep, gap.length=gap.length, gap.symbol=gap.symbol)
+	tmp <- pwMatrix(strings, sep = sep, gap.length = n-1)
 	SW <- tmp$M
 	segments <- tmp$rownames
 
-	# US: unigrams x segments
+	# NS: ngrams x segments (starting with unigrams)
 	tmp <- ttMatrix(segments)
-	US <- tmp$M
-	unigrams <- tmp$rownames
+	NS <- tmp$M
+	ngrams <- tmp$rownames
 
-	# Bisymbols x Segments
-	if (bigrams) {
-		
-		# remove gap character from US and symbols
-		if (length(strings) > 1) {
-			gap.char <- which(unigrams == gap.symbol)
-			unigrams <- unigrams[-gap.char]
-			US <- US[-gap.char,]
-		}
-		
-		S <- bandSparse(n = dim(US)[2], k = -1)
-		tmp <- rKhatriRao(US %*% S, US, unigrams, unigrams, binder = bigram.binder)
-		BS <- tmp$M
-		bisymbols <- tmp$rownames
-
-		# remove boundary from US and symbols
-		if (boundary) {
-			boundary.char <- which(unigrams == left.boundary | unigrams == right.boundary)
-			unigrams <- unigrams[-boundary.char]
-			US <- US[-boundary.char,]
-		}
-		
-	# various forms of output
-		if (simplify) {
-			result <- (BS*1) %*% (SW*1)
-			rownames(result) <- bisymbols
-			colnames(result) <- originals
-			return(result)
-		} else {
-			return(list(	segments = segments,
-		 					unigrams = unigrams,
-							bigrams = bisymbols,
-							SW = SW, # Segments x Words
-							US = US, # Unigrams x Segments
-							BS = BS # Bigrams x Segments
-							))
-		}
-	} else {
-		if (simplify) {
-			result <- (US*1) %*% (SW*1)
-			rownames(result) <- unigrams
-			colnames(result) <- strings
-			return(result)
-		}
-		return(list(	segments = segments,
-						unigrams = unigrams,
-						SW = SW, # Segments x Words
-						US = US # Unigrams x Segments
-						))
+	# larger ngrams
+	if (n > 1) {
+	  
+  	# remove gap character from ngrams
+  	remove <- which(ngrams == getOption("qlcMatrix.gap"))
+  	if (length(remove) > 0) {
+    	ngrams <- ngrams[-remove]
+    	NS <- NS[-remove,]
+  	}
+  	
+  	# remember unigrams
+	  US <- NS
+	  unigrams <- ngrams
+	  
+	  # construct ngrams
+	  for (i in 2:n) {
+		  S <- bandSparse(n = dim(NS)[2], k = i-1)
+		  tmp <- rKhatriRao(NS, US %&% S, ngrams, unigrams, binder = ngram.binder)
+		  NS <- tmp$M
+		  ngrams <- tmp$rownames
+	  }
 	}
+	
+	# remove gap symbol
+	remove <- which(segments == getOption("qlcMatrix.gap"))
+	if (length(remove > 0)) {
+	  segments <- segments[-remove]
+	  SW <- SW[-remove, ]
+	  NS <- NS[, -remove]
+	}
+	
+	# order ngrams according to ttMatrix locale
+	NN <- ttMatrix(ngrams)
+	NS <- NN$M %&% NS
+	ngrams <- NN$rownames
+	
+	# prepare result
+	if (simplify) {
+		result <- NS %*% SW
+		rownames(result) <- ngrams
+		colnames(result) <- originals
+	} else {
+		result <- list(	
+		  segments = segments,
+			ngrams = ngrams,
+			SW = SW, # Segments x Words
+			NS = NS # Unigrams x Segments
+			)
+	}
+	return(result)
 }
 
 # =========================================================
@@ -203,7 +200,8 @@ splitText <- function(	text,
 						lowercase = TRUE
 						) {
 
-	# make RunningWords x Verses, i.e. all words of the text in the order as they appear as rows, linked to the localSentenceIDs as columns.	
+	# make RunningWords x Verses, i.e. all words of the text in the order 
+  # as they appear as rows, linked to the localSentenceIDs as columns.	
 	tmp <- pwMatrix(text, sep = sep, gap.length = 0)
 	RS <- tmp$M
 	runningWords <- tmp$rownames
@@ -217,9 +215,9 @@ splitText <- function(	text,
 	# link the localSentenceIDs "S" to the globalSentenceIDs "U"
 	if (!is.null(globalSentenceID)) {
 		tmp <- jMatrix(localSentenceID, globalSentenceID)
-		US <- tmp$M1
+		US <- t(tmp$M2) %&% tmp$M1
 		# relink
-		RS <- RS %*% t(US)
+		RS <- RS %&% t(US)
 	}
 	
 	# remove upper/lowercase distinction for better statistics
@@ -233,7 +231,7 @@ splitText <- function(	text,
 
 	if (!lowercase) {
 		if (simplify) {
-			R <- (WR*1) %*% (RS*1)
+			R <- (WR) %*% (RS)
 			rownames(R) <- wordforms
 			return(R)
 		} else {
@@ -247,7 +245,7 @@ splitText <- function(	text,
 		}
 	} else {
 		if (simplify) {
-			R <- (wW*1) %*% (WR*1) %*% (RS*1)
+			R <- (wW) %*% (WR) %*% (RS)
 			rownames(R) <- lower
 			return(R)
 		} else {
@@ -273,9 +271,9 @@ splitWordlist <- function(	data,
 							doculects = "DOCULECT", 
 							concepts = "CONCEPT",
 							counterparts = "COUNTERPART",
-							splitstrings = TRUE,
+							ngrams = 2,
 							sep =  "",
-							bigram.binder = "",
+							ngram.binder = "",
 							grapheme.binder = "_",
 							simplify = FALSE
 							) {
@@ -302,72 +300,60 @@ splitWordlist <- function(	data,
 	WL <- tmp$M
 	words <- tmp$rownames
 	# split counterparts again from doculects
-	words <- sapply(strsplit(words,binder),head)[2,]
+	words <- sapply(strsplit(words, binder), head)[2,]
 
 	# relink	
-	DW <- DL %*% t(WL)
-	CW <- CL %*% t(WL)
+	DW <- DL %&% t(WL)
+	CW <- CL %&% t(WL)
 	
-	if (splitstrings) {
+	if (!is.null(ngrams)) {
 	
 		# split strings
-		S <- splitStrings(words, sep = sep, bigram.binder = bigram.binder)
+		S <- splitStrings(words, sep = sep, n = ngrams, ngram.binder = ngram.binder)
 	
 		# return results
 		if (simplify) {
 			
-			BW <- (S$BS*1) %*% (S$SW*1)
+			NW <- S$NS %*% S$SW
 			
 			# only use column names once because of size
 			rownames(DW) <- doculects
 			rownames(CW) <- concepts
-			rownames(BW) <- S$bigrams
-			colnames(BW) <- words
+			colnames(NW) <- words
+			rownames(NW) <- S$ngrams
 					
-			return(list(DW = DW, CW = CW, BW = BW))
+			return(list(DW = DW, CW = CW, NW = NW))
 			
 		} else {
 			# separate characters to languages
 			# and prepare full output (rather long!)
 			
 			# link to segments to doculects
-			DS <- DW %*% t(S$SW)
+			DS <- DW %&% t(S$SW)
 	
-			# Graphemes x Segments
-			tmp <- rKhatriRao(	DS, S$US, 
-								doculects, S$unigrams, 
+			# KhatriRao to turn N-symbols into 
+			# language-specific N-graphs x Segments
+			tmp <- rKhatriRao(	DS, S$NS, 
+								doculects, S$ngrams, 
 								binder = grapheme.binder
 								)
 			GS <- tmp$M
-			graphemes <- tmp$rownames
-	
-			# another KhatriRao to turn bisymbols into 
-			# language-specific Bigraphs x Segments
-			tmp <- rKhatriRao(	DS, S$BS, 
-								doculects, S$bigrams, 
-								binder = grapheme.binder
-								)
-			TS <- tmp$M
-			digraphs <- tmp$rownames
+			ngraphs <- tmp$rownames
 	
 			return(list(	doculects = doculects,
 							concepts = concepts,
 							words = words,
 						
 							segments = S$segments,
-							unigrams = S$unigrams,
-							bigrams = S$bigrams, 
-							graphemes = graphemes,
-							digraphs = digraphs,
+							ngrams = S$ngrams, 
+							ngraphs = ngraphs,
 	
 							DW = DW, # Doculects x Words
 							CW = CW, # Concepts x Words
 
 							SW = S$SW,	# Segments x Words
-							US = S$US,	# Unigrams x Segments
-							BS = S$BS,	# Bigrams x Segments
-							GS = GS, 	# Graphemes x Segments
-							TS = TS 	# Digraphs x Segments	
+							NS = S$NS,	# N-grams x Segments
+							GS = GS 	# N-graphs x Segments	
 							))
 		}
 	} else {
